capture program drop vulncalc
program define vulncalc, eclass


  # delimit;
  version 7.0;
  ** THIS CODE PREDICTS CONSUMPTION BASED ON THE LINEAR OR LOG CONSUMPTION
  ** PREDICTION EQUATION.;
  ** First we just get the syntax straight and name local macros.;
  ** i is household id variable
  ** t is time variable
  ** c is consumption variable (real per-capita)
  ** touse is 1 if vulnerability should be calculated for that household
    ** it is 1 iff, a)the household has an observation on consumption in
    ** each time period b) the household has positive consumption in every
    ** time period and c) the household was not `if'ed out in the user
    ** specified if command
  ** tousei is 1 if the household is not missing an observation in any time
    ** period on any of the idiosyncratic risk variables and if touse=1
    ** as well. If tousei is 0 idiosyncratic risk and unexplained risk will
    ** not be calculated for that household;
  ** ir`num' (i.e. ir1, ir2, ir3, etc.) are the idiosyncratic risk variables;
  ** `ir_n' is the number of idiosyncratic risk variables.;
  syntax varlist(min=5 default=none) [pweight] [, gamma(real 2) 
    z(real 0) ir_n(integer 0) runreg (real 0) runregvar(varlist)];
  tokenize `varlist';
  local i `1';
  local t `2';
  local c `3';
  local touse `4';
  local tousei `5';
  macro shift 5;
  forvalues num= 1/`ir_n' {;
    local ir`num' ``num'';
  };
  macro shift `ir_n';
  tokenize `exp';
  local w `2';
  if "`w'"=="" {;
    tempvar w;
    quietly gen `w'=1;
  };


  ** The user specifies the value of z in the ,z(#) option.
  ** The default is 0.
  ** If z=0 we normalize consumption, dividing by average 
  ** (over time periods and over households) consumption.;
  ** If z~=0 we normalize consumption, dividing by the poverty line.;
  ** c1 is log consumption.
  ** Ref is the reference consumption used in the calculation.
  ** If all households had normalized consumption= Ref there would be no
  ** poverty, no risk, and no vulnerability in the population.;
  

  tempvar c1 ElC;
  tempname e;
  sort `touse';
  quietly gen `c1'=log(`c');
  quietly by `touse': gen `ElC'=sum(`c1'*`w')/sum(`w') if `touse'==1;
  quietly by `touse': replace `ElC'=`ElC'[_N] if `touse'==1;


  ** Now we predict each households expected consumption linearly
  ** as the households average consumption.
  ** The prediction will be EC.;
  ** CCh will be EC minus ELC i.e. the mean-0 household effect.;
  tempvar EC CCh;
  quietly egen `EC'= mean(`c1') if `touse'==1, by(`i');
  quietly gen `CCh'=`EC'-`ElC';

  ** Next we want to make the predictions of consumption based on
  ** aggregate shocks. We do that by calculating average consumption
  ** in each month, minus overall average consumtion.
  ** ET is average consumption by month.
  ** CCt is average consumption by month minus overall average
  ** consumption, i.e. the mean-0 time period effect.;
  tempvar CCt ET;
  sort `touse' `t';
  quietly by `touse' `t': gen `ET' = sum(`c1'*`w')/sum(`w') if
    `touse'==1;
  quietly by `touse' `t': replace `ET'=`ET'[_N] if `touse'==1;
  quietly gen `CCt'=`ET' - `ElC';

  ** Now we will orthogonalize each idiosyncratic risk variable
    ** (minus its time mean and minus its household mean
    ** and plus its overall mean).
  ** We first take out the time and houshold means and add
    ** in the overall mean to each idiosyncratic risk variable.;
  ** ir`num' are the original idiosyncratic risk variables.;
  ** irnew`num' will be the new mean-0 idiosyncratic risk variables.;
  forvalues num= 1/`ir_n' {;
    sort `tousei' `t';
    tempvar irnew`num' r`num' ECgI`num' mean`num' j`num' t`num';
    quietly gen `irnew`num''=`ir`num'' if `tousei'==1;
    quietly by `tousei': gen `mean`num'' = sum(`irnew`num''*`w') /
      sum(`w') if `tousei'==1;
    quietly by `tousei': replace `mean`num''=`mean`num''[_N] if
      `tousei'==1;
    quietly egen `j`num''=mean(`irnew`num'') if `tousei'==1, by (`i');
    quietly by `tousei' `t': gen `t`num''=sum(`irnew`num''*`w') /
      sum(`w') if `tousei'==1;
    quietly by `tousei' `t': replace `t`num''=`t`num''[_N] if
      `tousei'==1;
    quietly replace `irnew`num''=`irnew`num''-`j`num''
      -`t`num'' +`mean`num'' if `tousei'==1;
  };



  ** Now we do the orthogonalization.
  ** We do this by regressing the id risk var #2 on idrisk var #1,
    ** save resid r2;
  ** Then we regress id risk var #3 on idrisk var #2 and idriskvar #1, 
    ** save resid r3;
  ** and we go on like this until we have completed it 
  ** for all the idiosyncratic risk variables.;
  forvalues num= 2/`ir_n' {;
    local it1=`num'-1;
    quietly reg `irnew`num'' `irnew1'-`irnew`it1'' [`weight' `exp'];
    quietly predict `r`num'', resid; 
  };

 
  ** Now comes the prediction equation:
  ** We regress Y =hh cons minus overall ave cons on 
    ** A) time mean of cons - overall mean
    ** B) hh mean of cons - overall mean
    ** C) decomposed orthogonal version of idiosyncratic risk
    **    variables.;
  ** and save the resulting coefficients.;
  tempvar Y res u;
  tempname Beta ;
  quietly gen `Y'=`c1'-`ElC';
  if `ir_n'==0 {;
    quietly reg `Y' `CCh' `CCt' [`weight' `exp'];
    matrix `Beta'= e(b);
    quietly predict `res', resid;   
    quietly gen `u' = `res' + `ElC';
  };
  else if `ir_n'==1 {;
    quietly reg `Y' `CCh' `CCt' `irnew1' [`weight' `exp'];
    matrix `Beta'= e(b);
    quietly predict `res', resid;
    quietly gen `u' = `res' + `ElC';
  };
  else if `ir_n'>1 {;
    quietly reg `Y' `CCh' `CCt' `irnew1' `r2'- `r`ir_n'' [`weight'
      `exp'];
    matrix `Beta'= e(b);
    quietly predict `res', resid;
    quietly gen `u' = `res' + `ElC';
  };


  ** We save coefficients BetaA BetaB BetaC1 BetaC2 etc. from this
  ** regression. Note that BetaA and BetaB will be equal to 1 because
  ** they are the fixed time and household effects;


  ** Third, we calculate expected consumptions based on each of these
  ** things one at a time.
  ** The variables are ECgH--expected consumption given household mean.
  ** ECgT--expected consumption given household and time mean.
  ** ECgI1--expected consumption given household and time mean and 
  ** first idiosyncratic risk variable. etc. etc.;
  tempvar ECgH ECgT ECgZECt ECgZECh test;
  quietly generate `ECgZECh'=exp(`CCh');
  quietly generate `ECgZECt'=exp(`CCt');
  forvalues num= 1/`ir_n' {;
    tempvar ECgZECi`num';
    if `num'==1 {;
      quietly generate `ECgZECi1'=exp(`Beta'[1,3]*`irnew1');
    };
    else if `num' ~=1 {;
      local it1=`num'-1;
      quietly generate `ECgZECi`num''=exp(`Beta'[1,`num'+2]*`r`num'');
    };
  };

  ** Create products of idiosyncratic risk part.;
  local irnp=`ir_n'+1;
  tempvar EC0m EC`irnp' testT testH;
  quietly generate `EC0m'=1;
  quietly generate `EC`irnp''=1;
  forvalues num= 1/`ir_n' {;
    tempvar EC`num'm;
    local it1=`num'-1;  
    quietly gen `EC`num'm'=`EC`it1'm' * `ECgZECi`num'';
    local back2=`ir_n'-`num'+2;
    local back1=`ir_n'-`num'+1;
    tempvar EC`back1';    
    quietly gen `EC`back1''=`EC`back2'' * `ECgZECi`back1'';
  };

  quietly egen `testT'=  mean(`EC`ir_n'm' * `ECgZECt' * exp(`u')); 
  quietly generate `ECgH'= `testT' * `ECgZECh';
  quietly egen `testH'=  mean(`EC`ir_n'm' * exp(`u'));
  quietly generate `ECgT'=  `testH' * `ECgZECt' * `ECgZECh';
  forvalues num= 1/`ir_n' {;
    if `num' <`ir_n' {;
      local it1=1+`num';
      tempvar test`num';
      quietly egen `test`num''=  mean(`EC`it1'' * exp(`u'));
      quietly generate `ECgI`num''=`test`num''*`EC`num'm'*`ECgZECt'*`ECgZECh';
    };
    else if `num'==`ir_n' {;
      local it1=1+`num';
      tempvar test`num';
      quietly egen `test`num''=  mean(exp(`u'));
      quietly generate `ECgI`num''=`test`num''*`EC`num'm'*`ECgZECt'*`ECgZECh';
    };
  };



  tempvar PL;
  if `z'==0 {;
    sort `touse';
    quietly by `touse': gen `PL'=sum(`c'*`w')/sum(`w') if `touse'==1;
    quietly by `touse': replace `PL'=`PL'[_N] if `touse'==1;
  };
  else if `z'>0 {;
    quietly gen `PL'=`z';
  };
  quietly replace `c'=`c'/`PL' + .1;
  quietly replace `ECgH'=`ECgH'/`PL' + .1;
  quietly replace `ECgT'=`ECgT'/`PL' + .1;
  forvalues num=1/`ir_n' {;
    tempvar ECgII`num';
    quietly generate `ECgII`num''=`ECgI`num''/`PL' + .1;
  };
  local ref=1.1;


  ** This calls on vulncalc2, which is the code that takes the consumption
  ** predictions and calculates vulnerability, poverty, and risk measures.;
  if `runreg'==0 {;
    if `ir_n'>0 {;
      vulncalc2 `i' `t' `c' `touse' `tousei' `ECgH' `ECgT'
        `ECgII1'-`ECgII`ir_n'' [`weight' `exp'], gamma(`gamma') 
        ir_n(`ir_n') ref(`ref');
    };
    else if `ir_n'==0 {;
      vulncalc2 `i' `t' `c' `touse' `tousei' `ECgH' `ECgT'
        [`weight' `exp'], gamma(`gamma') ir_n(`ir_n') ref(`ref');
    };
  };
  else if `runreg'==1 {;
    if `ir_n'>0 {;
      vulncalc2 `i' `t' `c' `touse' `tousei' `ECgH' `ECgT'
        `ECgII1'-`ECgII`ir_n'' [`weight' `exp'], gamma(`gamma') ir_n(`ir_n')
        runreg (1) runregvar(`runregvar') ref(`ref');
    };
    else if `ir_n'==0 {;
      vulncalc2 `i' `t' `c' `touse' `tousei' `ECgH' `ECgT'
        [`weight' `exp'], gamma(`gamma') ir_n(`ir_n')
        runreg (1) runregvar(`runregvar') ref(`ref');
    };
  };
end;

