capture program drop vulncalc2
program define vulncalc2, eclass


  # delimit;
  version 7.0; 
  ** VULNCALC2 USES THE CONSUMPTION PREDICTIONS FROM VULNCALC
  ** TO CALCULATE VULNERABILITY, POVERTY, AND RISK VARIABLES.
  ** First we just get the syntax straight and name local macros.;
  ** i is household id variable
  ** t is time variable
  ** c1 is normalized consumption variable (real per-capita)
  ** touse is 1 if vulnerability should be calculated for that household
    ** it is 1 iff, a)the household has an observation on consumption in
    ** each time period b) the household has positive consumption in every
    ** time period and c) the household was not `if'ed out in the user
    ** specified if command
  ** tousei is 1 if the household is not missing an observation in any time
    ** period on any of the idiosyncratic risk variables and if touse=1
    ** as well. If tousei is 0 idiosyncratic risk and unexplained risk will
    ** not be calculated for that household;
  ** ECgH is predicted consumption based on hh identity.
  ** ECgT is predicted consumption based on hh identity and aggregate shocks.
  ** ECg1-N are predicted consumptions based on hh identity, aggregate
    ** shocks, and each previous idiosyncratic risk variable.
  ** `ir_n' is the number of idiosyncratic risk variables.;
  ** `ref' is reference utility, (i.e. 1).;
  syntax varlist(min=7 default=none) [pweight] [, gamma(real 2) 
    ir_n(integer 0) runreg(real 0) runregvar(varlist) ref(real 1)];
  tokenize `varlist';
  local i `1';
  local t `2';
  local c1 `3';
  local touse `4';
  local tousei `5';
  local ECgH `6';
  local ECgT `7';
  macro shift 7;
  forvalues num= 1/`ir_n' {;
    local ECgI`num' ``num'';
    tempvar UECgI`num' EUECgI`num';
    global i`num' "idrisk`num'";
  };

**************************************************************************;
**************************************************************************;
****THE SECOND HALF OF THE CODE USES THE CONSUMPTION PREDICTIONS**********;
****ESTIMATED IN THE FIRST HALF OF THE CODE*******************************;
****TO CALCULATE THE VULNERABILITY MEASURES*******************************;
**************************************************************************;
**************************************************************************;

  ** We can now start estimating vulnerability over the relevant households.
  ** We will now calculate (Uref), which is either utility from the
  ** reference consumption. Uref =U(ref);
  ** Because of that, when gamma=2, we can interpret vulnerability as 
  ** the percent utility loss due to vulnerability.;
  tempname Uref;
  if `gamma'~=1 {;
    scalar `Uref'=(`ref'^(1-`gamma'))/(1-`gamma');
  };
  else if `gamma'==1 {;
    scalar `Uref'=ln(`ref');
  };


  ** We will now calculate expected utility (EUC) for each household.;
  ** This is the average of their actual utilities (UC) in each time period.;
  tempvar UC EUC;
  if `gamma'==1 {;
    quietly generate `UC'=ln(`c1');
    quietly egen `EUC'= mean(`UC') if `touse'==1, by(`i');
  };
  else if `gamma' ~=1 {;
    quietly generate `UC'=(`c1'^(1-`gamma'))/(1-`gamma');
    quietly egen `EUC'=mean(`UC') if `touse'==1, by(`i');
  };


  ** Ligon-Schechter define vulnerability as [(Uref-UEC)+(UEC-EUC)].;
  ** Where the first part is poverty and the second is risk.;
  ** We can now calculate vulnerability.;
  global v "vulnerability";
  quietly gen $v = `Uref'-`EUC';


  ** Now we will calculate poverty, risk, aggregate risk, idiosyncratic 
  ** risk, and unexplained risk.;
  ** To do so we calculate utility of predicted consumption given 
  ** each new piece of information.;
  tempvar UECgH UECgT;
  if `gamma'==1 {;
    quietly gen `UECgH'=ln(`ECgH');
    quietly gen `UECgT'=ln(`ECgT');
    forvalues num= 1/`ir_n' {;
      quietly gen `UECgI`num''=ln(`ECgI`num'');
    };
  };
  if `gamma'~=1 {;
    quietly gen `UECgH'=(`ECgH'^(1-`gamma'))/(1-`gamma');
    quietly gen `UECgT'=(`ECgT'^(1-`gamma'))/(1-`gamma');
    forvalues num= 1/`ir_n' {;   
      quietly gen `UECgI`num''=(`ECgI`num''^(1-`gamma'))/(1-`gamma');
    };
  };

  ** Then we calculate expected utility of predicted consumption
  ** given each new piece of information.;
  tempvar EUECgH EUECgT;
  quietly egen `EUECgH'=mean(`UECgH') if `touse'==1, by(`i');
  quietly egen `EUECgT'=mean(`UECgT') if `touse'==1, by(`i');
  forvalues num= 1/`ir_n' {;
    quietly egen `EUECgI`num''=mean(`UECgI`num'') if `touse'==1, by(`i');
  };


  ** Then we calculate each component of risk
  ** by calculating the difference of expected utilities
  ** given each new piece of information.;
  global p "poverty";
  global r "risk";
  global a "agrisk";
  global u "unexprisk";
  quietly gen $p = `Uref'-`EUECgH';
  quietly gen $r = `EUECgH' - `EUC';
  quietly gen $a=`EUECgH'-`EUECgT';
  if `ir_n'>0 {;
    quietly gen $i1=`EUECgT'-`EUECgI1';
    if `ir_n'>1{;
      local it1=1;
      forvalues num= 2/`ir_n' {;
        quietly gen ${i`num'}=`EUECgI`it1''-`EUECgI`num'';
        local it1=`it1'+1;
      };
    };
    quietly gen $u=`EUECgI`ir_n''-`EUC';
  };
  else if `ir_n'==0 {;
    quietly gen $u=`EUECgT'-`EUC';
  };
  global it "idrisktot";
  if `ir_n'>0 {;
    quietly egen $it=rsum($i1-${i`ir_n'}) if `tousei'==1;
  };
  else if `ir_n'==0 {;
     quietly gen $it=0 if `touse'==1;
  };


  ** This part will run the regression if the user requests to do so.
  ** Remember that there will only be one observation for each household.
  ** Thus, the first-period obs. of any time-varying variable will be used
  ** on the left hand side.;
  if `runreg'==1 {;
    local iterate=0;
    global runregBetav "runregBetav";
    global runregBetap "runregBetap";
    global runregBetar "runregBetar";
    global runregBetaa "runregBetaa";
    global runregBetait "runregBetait";
    global runregBetau "runregBetau";
    global r2v "r2v";
    global r2p "r2p";
    global r2r "r2r";
    global r2a "r2a";
    global r2it "r2it";
    global r2u "r2u";
    global Nv "Nv";
    global Ni "Ni";
    foreach var of varlist `runregvar' {;
      local iterate=`iterate'+1;
      global namerrv`iterate' "`var'";
    };
    quietly reg $v `runregvar' if `t'==1 [`weight'`exp'];
    matrix $runregBetav=e(b);
    scalar $r2v=e(r2);
    scalar $Nv=e(N);
    quietly reg $p `runregvar' if `t'==1 [`weight'`exp'];
    matrix $runregBetap=e(b);
    scalar $r2p=e(r2);
    quietly reg $r `runregvar' if `t'==1 [`weight'`exp'];
    matrix $runregBetar=e(b);
    scalar $r2r=e(r2);
    quietly reg $a `runregvar' if `t'==1 [`weight'`exp'];
    matrix $runregBetaa=e(b);
    scalar $r2a=e(r2);
    quietly reg $it `runregvar' if `t'==1
      [`weight'`exp'];
    matrix $runregBetait=e(b);
    scalar $r2it=e(r2);
    quietly reg $u `runregvar' if `t'==1
      [`weight'`exp'];
    matrix $runregBetau=e(b);
    scalar $r2u=e(r2);
    forvalues num= 1/`ir_n' {;
      global runregBetai`num' "runregBetai`num'";
      global r2i`num' "r2i`num'";
      quietly reg ${i`num'} `runregvar' if `t'==1
        [`weight'`exp'];
      matrix ${runregBetai`num'}=e(b);
      scalar ${r2i`num'}=e(r2);
    };
    scalar $Ni=e(N);
  };
end;

