capture program drop vulnerability
program define vulnerability, eclass sortpreserve


  # delimit;
  version 7.0;
  ** The panel data should be in `long' form (see help reshape);
  ** This code will work with an unbalanced panel,;
  ** but will ignore those households with missing consumption variables.;
  ** It will also ignore households with negative or zero consumption;
  ** in any period.;
  ** You are allowed any # of variables which represent idosyncratic risk.
  ** These variables must vary over time, and if you are including
  ** a variable such as income it should be in real per capita terms already.;
  ** If there is a household with missing data for the idiosyncratic risk
  ** variable, it will be excluded from the calculation of idiosyncratic
  ** and unexplained risk, but will still be included in the calculation 
  ** of vulnerability, poverty, risk, and aggregate risk.
  ** Please note that this code creates variables called vulnerability,;
  ** poverty, risk, agrisk, idrisktot, unexprisk and idrisk1-N.;
  ** If you already have a variable with one of those names;
  ** the program will say, e.g. "vulnerability already defined";
  ** If you would like to calculate vulnerability twice, for example;
  ** once using total consumption and once using food consumption,;
  ** You should run it the first time and then rename the output variables;
  ** before running it again.;


  syntax varlist(min=3 default=none) [pweight] [if]
   [, gamma(real 2) z(real 0) boot(integer 0)
   nreps(integer 1000) runregvar(varlist)];
  tokenize `varlist';
  local i `1';
  local t `2';
  local c `3';
  macro shift 3;
  local  ir `*';
  tokenize `exp';
  local w `2';
  if "`w'"=="" {;
    tempvar w;
    quietly gen `w'=1;
  };

  ** The arguments for this function are, in that order,
  ** i (household id variable name) t (time period variable name)
  ** c (consumption variable name), followed by ir (any idiosyncratic
  ** risk variables).
  ** Gamma (the constant of relative risk aversion)
  ** and z(whether vulnerability will be calculated with respect to
  ** a poverty line, or with respect to average consumption) are also inputs.
  ** Consumption and income variables must be in real per-capita terms.
  ** The default for gamma is 2,
  ** but real numbers greater than or equal to 1 are permited.
  ** For gamma=1 the utility function is log utility.
  ** For gamma~=1 the utility function is CRS with CRRA of gamma.
  ** For z=0 average consumption is used as the reference point.
  ** Thus relative vulnerability is calculated.
  ** For z~=0, z is the poverty line.
  ** The default for z is 0.
  ** pweights are allowed, w is the sampling weight.
  ** For boot=1, a bootstrap will be done, for boot=0 it will not.
  ** The default is boot=0.
  ** nreps is the number of repetitions for a bootstrap.
  ** The default is nreps=1000.
  ** If runregvar is specified a regression will be run and
  ** the variables in runregvar will be the RHS variables.;

  ** This part will make sure that the numbers input by the user
  ** are appropriate.;
  if `gamma'<0 {;
    display as error "The constant of relative risk aversion" _continue;
    display " must be at least 0 or higher.";
    error 459;
  };
  if `z'<0 {;
    display as error "z must be either 0 or positive.";
    error 459;
  };
  if `nreps'<=0 {;
    display as error "The number of repetitions must be" _continue;
    display " an integer greater than 0.";
    error 459;
  };
  if `boot'~=0 & `boot'~=1 {;
    display as error "boot must be 0 or 1";
    error 459;
  };
  if "`runregvar'"~="" {;
    local runreg=1;
  };
  else if "`runregvar'"=="" {;
    local runreg=0;
  };


  ** This part of the code will find those households with negative
  ** consumption in any period and those with missing variables;
  ** It does so by temporarily converting the data to wide format
  ** and finding the observations with missing variables.;
  ** `missingc' are those households with missing consumption data.
  ** `missingi' are those households without missing id. risk data.
  ** `negg' are those households with negative cons. in some period.
  ** `useriff' are those households specified to be used by the user
    ** in his or her if command.;
  ** The part with `userif' and `useriff' gets a bit complicated because
  ** it will not only allow the user to specify a prerequesite such as
  ** "if age<50" (the `useriff' part of the commands). It will also allow
  ** the user to specify a prerequesite such as "if month~=12" 
  ** (which gives the `userif' parts of the commands).; 
  tempname N;
  tempvar neg negg userif useriff fake_t missingc missingi;
  quietly egen `fake_t'=group(`t') `if';
  quietly summ `fake_t';
  local fake_t_max=r(max);
  gen `neg'=0;
  gen `userif'=0;
  quietly replace `userif'=1 `if';
  egen `useriff'=sum(`userif'), by(`i');
  quietly replace `neg'=1 if `c' <=0 & `useriff'==`fake_t_max';
  egen `negg' =sum(`neg'), by(`i');
  preserve;
  drop `fake_t';
  quietly egen `fake_t'=group(`t');
  keep `c' `i' `fake_t' `ir';
  quietly reshape wide `c' `ir', i(`i') j(`fake_t'); 
  gen `missingc'=0;
  gen `missingi'=0;
  forvalues num= 1/`fake_t_max' {;               
    quietly replace `missingc' =1 if `c'`num'==.;
    foreach var in `ir' {;
      quietly replace `missingi'=1 if `var'`num'==.;
    };
  };
  keep `i' `missingc' `missingi';
  sort `i';
  quietly summ `i';
  scalar `N'=r(N);
  tempfile file1;
  quietly save `file1';
  restore;
  sort `i';
  merge `i' using `file1';
  drop _merge;


  ** This marks out those households with negative consumption
  ** or unbalanced data over the relevant variable set;
  ** It then informs the user how many observations will be ignored.;
  ** For hh's with touse=1, vuln, pov, and agg risk will be calculated.;
  ** For hh's with tousei=1 as well, idiosyncratic and unexplained risk will 
  ** be calculated as well.;  
  tempvar touse tousei;
  gen `touse'=(`negg'==0 & `missingc'==0);
  tempvar touse_hh;
  tempname k_touse;  
  egen `touse_hh'=sum(`touse'), by(`t');
  quietly summ `touse_hh';
  scalar `k_touse'=r(min);
  scalar `k_touse'=`N'-`k_touse';
  display "";
  if `k_touse'>0 {;
    display "";
    display as text "We will ignore the " _continue;
    display as error `k_touse' as text " households" _continue;
    display " with negative or 0 consumption or";
    display "missing data for consumption in at least one time period."; 
  };
  else {;
    display "";
    display as text "We will calculate a vulnerability measure for";
    display "every household, as it is a balanced panel";
    display "and there are no observations with negative consumption.";
  };
  quietly replace `touse'=(`negg'==0 & `missingc'==0 
    & `useriff'==`fake_t_max' & `userif'==1);
  quietly gen `tousei'=(`missingi'==0 & `touse'==1);


  ** Here we count how many idiosyncratic risk variables (ir) were entered.;
  local iterate=0;
  if "`ir'"~="" {;
    foreach var of varlist `ir' {;
      local iterate=`iterate'+1;
      tempvar ee`iterate';
      tempname im`iterate';
    };
  };
  local ir_n=`iterate';
  

  ** We will now run the bootstrap if that was requested.;
  if `boot'==1 {;
    ** Here we are opening the postfiles to post the bootstrap results to.;
    tempname vm;
    tempfile bootstrap;
    postfile `vm' vulnmean povmean riskmean agriskmean idrisktotmean 
      unexpriskmean using `bootstrap';
    forvalues num= 1/`ir_n' {;
      tempfile bootstrap`num'; 
      postfile `im`num'' idrisk`num'mean using `bootstrap`num'';
    };
    if `runreg'==1 {;
      local iterate=0;
      foreach var of varlist `runregvar' {;
        local iterate=`iterate'+1;
        tempfile regboot`iterate';
        tempname rrvbeta`iterate';
        postfile `rrvbeta`iterate'' rrbeta`iterate'v rrbeta`iterate'p 
          rrbeta`iterate'r rrbeta`iterate'a rrbeta`iterate'it 
          rrbeta`iterate'u using `regboot`iterate''; 
        forvalues num= 1/`ir_n' {;
          tempfile regboot`iterate'i`num';
          tempname rrvbeta`iterate'i`num';
          postfile `rrvbeta`iterate'i`num'' rrbeta`iterate'i`num'
            using `regboot`iterate'i`num'';
        }; 
      };
      tempname rrvbetaconst;
      tempfile regbootconst;
      postfile `rrvbetaconst' rrbetaconstv rrbetaconstp rrbetaconstr 
        rrbetaconsta rrbetaconstit rrbetaconstu using `regbootconst';
      forvalues num= 1/`ir_n' {;
        tempname rrvbetaconsti`num';
        tempfile regbootconsti`num';
        postfile `rrvbetaconsti`num'' rrbetaconsti`num'
          using `regbootconsti`num'';
      };
    };

    local st="$S_TIME";
    ** Here we actually run the bootstrap. The computer will count out
    ** every 50 repetitions as well as displaying the elapsed time
    ** between (st) over each 50 repetitions.;
    tempvar aa bb cc dd ee ff;
    display as text "";
    display "The computer will count out every 50 repetitions...";
    preserve;
    forvalues num= 1/`nreps' {;
      local in=`num'/50;
      local inl1=inlist(`in',1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,
        19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
        41,42,43,44,45,46,47,48,49,50,61,62,63,64,65,66,67,68,69,70,71,72,
        73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94);
      local inl2=inlist(`in',95,96,97,98,99,100,101,102,103,104,105,106,
        107,108,109,110,111,112, 113,114,115,116,117,118,119,120,121,122,123,
        124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,
        141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157);
      local inl3=inlist(`in',158,159,160,161,162,163,164,165,166,167,168,169,
        170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,
        187,188,189,190,191,192,193,194,195,196,197,198,199,200);
      if `inl1'==1 | `inl2'==1 |`inl3'==1 {;
        display "Repetition #" `num';
      elapse `st'; local st="$S_TIME";
      };
      quietly drop if `touse'==0;
      bsample, cluster(`i');
      if `runreg'==0 {;
        vulncalc `i' `fake_t' `c' `touse' `tousei' `ir' [`weight' `exp'],
          gamma(`gamma') z(`z') ir_n(`ir_n') runreg(0);
      };
      else if `runreg'==1 {;
        vulncalc `i' `fake_t' `c' `touse' `tousei' `ir' [`weight' `exp'],
          gamma(`gamma') z(`z') ir_n(`ir_n') 
          runreg(1) runregvar(`runregvar');
      };
      ** This part calculates the average vulnerability measures
      ** in the bootstrapped sample, and the beta coefficients given 
      ** by the bootstrapped sample and posts them to the postfile.;
      sort `tousei';
      quietly gen `aa' = sum($v*`w')/sum(`w') if $v~=.;
      local aaa=`aa'[_N];
      quietly gen `bb' = sum($p*`w')/sum(`w') if $p~=.;
      local bbb=`bb'[_N];
      quietly gen `cc' = sum($r*`w')/sum(`w') if $r~=.;
      local ccc=`cc'[_N];
      quietly gen `dd' = sum($a*`w')/sum(`w') if $a~=.;
      local ddd=`dd'[_N];
      quietly gen `ee' = sum($it*`w')/sum(`w') if $it~=.;
      local eee=`ee'[_N];
      quietly gen `ff' = sum($u*`w')/sum(`w') if $u~=.;
      local fff=`ff'[_N];
      post `vm' (`aaa') (`bbb') (`ccc') (`ddd') (`eee') (`fff');
      forvalues num2= 1/`ir_n' {;
        tempvar `ee`num2'';
        quietly gen `ee`num2'' = sum(${i`num2'}*`w')/sum(`w') if ${i`num2'}~=.;
        local eee`num2'=`ee`num2''[_N];
        post `im`num2'' (`eee`num2'');
        drop ${i`num2'};
      };
      if `runreg'==1 {;
        local rrv_n=colsof($runregBetav)-1;
        forvalues num2=1/`rrv_n' {;
          tempname rrb`num2'v rrb`num2'p rrb`num2'r rrb`num2'a rrb`num2'it
            rrb`num2'u;
          scalar `rrb`num2'v' = el($runregBetav,1,`num2');
          scalar `rrb`num2'p' = el($runregBetap,1,`num2');
          scalar `rrb`num2'r' = el($runregBetar,1,`num2');
          scalar `rrb`num2'a' = el($runregBetaa,1,`num2');
          scalar `rrb`num2'it' = el($runregBetait,1,`num2');
          scalar `rrb`num2'u' = el($runregBetau,1,`num2');
          post `rrvbeta`num2'' (`rrb`num2'v') (`rrb`num2'p') (`rrb`num2'r')
            (`rrb`num2'a') (`rrb`num2'it') (`rrb`num2'u');
          scalar drop `rrb`num2'v' `rrb`num2'p' `rrb`num2'r' `rrb`num2'a'
            `rrb`num2'it' `rrb`num2'u';
          forvalues num3= 1/`ir_n' {;
            tempname rrb`num2'i`num3';
            scalar `rrb`num2'i`num3'' = el(${runregBetai`num3'}, 1, `num2');
            post `rrvbeta`num2'i`num3'' (`rrb`num2'i`num3'');
            scalar drop `rrb`num2'i`num3'';           
          };
        };
        tempname rrbconstv rrbconstp rrbconstr rrbconsta rrbconstit rrbconstu;
        scalar `rrbconstv'=el($runregBetav,1,`rrv_n'+1);
        scalar `rrbconstp'=el($runregBetap,1,`rrv_n'+1);
        scalar `rrbconstr'=el($runregBetar,1,`rrv_n'+1);
        scalar `rrbconsta'=el($runregBetaa,1,`rrv_n'+1);
        scalar `rrbconstit'=el($runregBetait,1,`rrv_n'+1);
        scalar `rrbconstu'=el($runregBetau,1,`rrv_n'+1);
        post `rrvbetaconst' (`rrbconstv') (`rrbconstp') (`rrbconstr')
          (`rrbconsta') (`rrbconstit') (`rrbconstu') ;
        forvalues num3= 1/`ir_n' {;
          tempname rrbconsti`num3';
          scalar `rrbconsti`num3''=el(${runregBetai`num3'},1,`rrv_n'+1);
          post `rrvbetaconsti`num3'' (`rrbconsti`num3'');
          scalar drop `rrbconsti`num3'';
          matrix drop ${runregBetai`num3'};
        };
        matrix drop $runregBetav $runregBetap $runregBetar $runregBetaa 
          $runregBetait $runregBetau;
        scalar drop `rrbconstv' `rrbconstp' `rrbconstr' `rrbconsta'
          `rrbconstit' `rrbconstu' `num3';
      };
      drop $v $p $r $a $it $u  `aa' `bb' `cc' `dd' `ee' `ff';
      restore, preserve;
    };
    restore, not;

    ** And then we close the postfiles.;
    postclose `vm';
    forvalues num= 1/`ir_n' {;
      postclose `im`num'';
    };
    if `runreg'==1 {;
      forvalues num= 1/`rrv_n' {;
        postclose `rrvbeta`num'';
        forvalues num2=1/`ir_n' {;
          postclose `rrvbeta`num'i`num2'';
        };
      };
      postclose `rrvbetaconst';
      forvalues num2=1/`ir_n' {;
        postclose `rrvbetaconsti`num2'';
      };
    };
  };

 
  ** We will now call up the program which actually estimates vulnerability
  ** on the true sample.;
  preserve;
  tempfile newvars;
  if `runreg'==0 {;
    vulncalc `i' `fake_t' `c' `touse' `tousei' `ir' [`weight' `exp'],
      gamma(`gamma') z(`z') ir_n(`ir_n') runreg(0);
  };
  else if `runreg'==1 {;
    vulncalc `i' `fake_t' `c' `touse' `tousei' `ir' [`weight' `exp'],
      gamma(`gamma') z(`z') ir_n(`ir_n') runreg(1) runregvar(`runregvar');
  };
  if `ir_n'>0 {;
    keep `i' $v $p $r $a $it $i1-${i`ir_n'} $u;
  };
  else if `ir_n'==0 {;
    keep `i' $v $p $r $a $it $u;
  };
  sort `i';
  quietly save `newvars';
  restore;
  sort `i';
  merge `i' using `newvars';
  drop _merge;

  
  display "";
  display "";
  display as text "Means not taking into account sampling weights, even if specified";
  if `ir_n'>0 {;
    summ $v $p $r $a $it $i1-${i`ir_n'} $u;
  };
  else if `ir_n'==0 {;
    summ $v $p $r $a $it $u;  
  };
  preserve;

  ** And all the rest of the code is used to display the results nicely
  ** and has no economic content.;
  if `boot'==1 {;
    sort `tousei';
    tempvar realv realp realr reala realit realu;
    quietly gen `realv' = sum($v*`w')/sum(`w') if $v~=.;
    local realvv=`realv'[_N];
    quietly gen `realp' = sum($p*`w')/sum(`w') if $p~=.;
    local realpp=`realp'[_N];
    quietly gen `realr' = sum($r*`w')/sum(`w') if $r~=.;
    local realrr=`realr'[_N];
    quietly gen `reala' = sum($a*`w')/sum(`w') if $a~=.;
    local realaa=`reala'[_N];
    quietly gen `realit' = sum($it*`w')/sum(`w') if $it~=.;
    local realitt=`realit'[_N];
    quietly gen `realu' = sum($u*`w')/sum(`w') if $u~=.;
    local realuu=`realu'[_N];
    forvalues num= 1/`ir_n' {;
      tempvar reali`num';
      quietly gen `reali`num'' = sum(${i`num'}*`w')/sum(`w') if ${i`num'}~=.;
      local realii`num'=`reali`num''[_N];
    };
  };
  if `runreg'==1 {;
    local rrv_n=colsof($runregBetav)-1;
    forvalues num= 1/`rrv_n' {;
      local realb`num'v=el($runregBetav,1,`num');
      local realb`num'p=el($runregBetap,1,`num');
      local realb`num'r=el($runregBetar,1,`num');
      local realb`num'a=el($runregBetaa,1,`num');
      local realb`num'it=el($runregBetait,1,`num');
      local realb`num'u=el($runregBetau,1,`num');
      forvalues num2=1/`ir_n' {;
        local realb`num'i`num2'=el(${runregBetai`num2'},1, `num');
      };
    };
    local num=`rrv_n'+1;
    local realbconstv=el($runregBetav,1,`num');
    local realbconstp=el($runregBetap,1,`num');
    local realbconstr=el($runregBetar,1,`num');
    local realbconsta=el($runregBetaa,1,`num');
    local realbconstit=el($runregBetait,1,`num');
    local realbconstu=el($runregBetau,1,`num');
    forvalues num2= 1/`ir_n' {;
      local realbconsti`num2'=el(${runregBetai`num2'},1,`num');  
    };
  };

  if `boot'==1 {;
    if `runreg'==1 {;
      local Nv=$Nv;
      local Ni=$Ni;
      local r2v=$r2v;
      local r2p=$r2p;
      local r2r=$r2r;
      local r2a=$r2a;
      local r2it=$r2it;
      local r2u=$r2u;
      forvalues num=1/`ir_n'{;
        local r2i`num'=${r2i`num'};
      };
    };
    clear;
    quietly use `bootstrap';
    forvalues num= 1/`ir_n' {; 
      merge using `bootstrap`num'';
      drop _merge;
    };
    sort vulnmean;
    gen id=_n;
    tempname n_1l n_1h n_5l n_5h n_10l n_10h;
    quietly summ id;
    local n_max=r(max);
    scalar `n_1l'=round((.005*`n_max'),1);
    scalar `n_1h'=round((.995*`n_max'),1);
    scalar `n_5l'=round((.025*`n_max'),1);
    scalar `n_5h'=round((.975*`n_max'),1);
    scalar `n_10l'=round((.05*`n_max'),1);
    scalar `n_10h'=round((.95*`n_max'),1);


    summ vulnmean if id==`n_1l', meanonly; 
    local vn_1l=r(mean); 
    summ vulnmean if id==`n_1h', meanonly;
    local vn_1h=r(mean);
    summ vulnmean if id==`n_5l', meanonly;
    local vn_5l=r(mean);
    summ vulnmean if id==`n_5h', meanonly;
    local vn_5h=r(mean);
    summ vulnmean if id==`n_10l', meanonly;
    local vn_10l=r(mean);
    summ vulnmean if id==`n_10h', meanonly;
    local vn_10h=r(mean);


    sort povmean;
    quietly replace id=_n;
    summ povmean if id==`n_1l', meanonly; 
    local pn_1l=r(mean); 
    summ povmean if id==`n_1h', meanonly;
    local pn_1h=r(mean);
    summ povmean if id==`n_5l', meanonly;
    local pn_5l=r(mean);
    summ povmean if id==`n_5h', meanonly;
    local pn_5h=r(mean);
    summ povmean if id==`n_10l', meanonly;
    local pn_10l=r(mean);
    summ povmean if id==`n_10h', meanonly;
    local pn_10h=r(mean);
 

    sort riskmean;
    quietly replace id=_n;
    summ riskmean if id==`n_1l', meanonly;
    local rn_1l=r(mean);
    summ riskmean if id==`n_1h', meanonly;
    local rn_1h=r(mean);
    summ riskmean if id==`n_5l', meanonly;
    local rn_5l=r(mean);
    summ riskmean if id==`n_5h', meanonly;
    local rn_5h=r(mean);
    summ riskmean if id==`n_10l', meanonly;
    local rn_10l=r(mean);
    summ riskmean if id==`n_10h', meanonly;
    local rn_10h=r(mean);


    sort agriskmean;
    quietly replace id=_n;
    summ agriskmean if id==`n_1l', meanonly;
    local an_1l=r(mean);
    summ agriskmean if id==`n_1h', meanonly;
    local an_1h=r(mean);
    summ agriskmean if id==`n_5l', meanonly;
    local an_5l=r(mean);
    summ agriskmean if id==`n_5h', meanonly;
    local an_5h=r(mean);
    summ agriskmean if id==`n_10l', meanonly;
    local an_10l=r(mean);
    summ agriskmean if id==`n_10h', meanonly;
    local an_10h=r(mean);


    sort idrisktotmean; 
    quietly replace id=_n;
    summ idrisktotmean if id==`n_1l', meanonly;
    local in_1l=r(mean);
    summ idrisktotmean if id==`n_1h', meanonly;
    local in_1h=r(mean);
    summ idrisktotmean if id==`n_5l', meanonly;
    local in_5l=r(mean);
    summ idrisktotmean if id==`n_5h', meanonly;
    local in_5h=r(mean);
    summ idrisktotmean if id==`n_10l', meanonly;
    local in_10l=r(mean);
    summ idrisktotmean if id==`n_10h', meanonly;
    local in_10h=r(mean);


    forvalues num= 1/`ir_n' {;
      sort idrisk`num'mean;
      quietly replace id=_n;
      summ idrisk`num'mean if id==`n_1l', meanonly;
      local i`num'n_1l=r(mean);
      summ idrisk`num'mean if id==`n_1h', meanonly;
      local i`num'n_1h=r(mean);
      summ idrisk`num'mean if id==`n_5l', meanonly;
      local i`num'n_5l=r(mean);
      summ idrisk`num'mean if id==`n_5h', meanonly;
      local i`num'n_5h=r(mean);
      summ idrisk`num'mean if id==`n_10l', meanonly;
      local i`num'n_10l=r(mean);
      summ idrisk`num'mean if id==`n_10h', meanonly;
      local i`num'n_10h=r(mean);
    };


    sort unexpriskmean; 
    quietly replace id=_n;
    summ unexpriskmean if id==`n_1l', meanonly;
    local un_1l=r(mean);
    summ unexpriskmean if id==`n_1h', meanonly;
    local un_1h=r(mean);
    summ unexpriskmean if id==`n_5l', meanonly;
    local un_5l=r(mean);
    summ unexpriskmean if id==`n_5h', meanonly;
    local un_5h=r(mean);
    summ unexpriskmean if id==`n_10l', meanonly;
    local un_10l=r(mean);
    summ unexpriskmean if id==`n_10h', meanonly;
    local un_10h=r(mean);


    display"";
    display"";
    display as text "Bootstrapped confidence intervals for" _continue;
    display " survey means of vulnerability measures.";
    display "These take into account sampling weights if specified.";
    display"";
    display in text "Variable{c |}Pop. Mean [ 90% Conf. Int."_continue;
    display "] [ 95% Conf. Int.] [ 99% Conf. Int.]";
    display in text "{hline 8}{c +}{hline 63}";
    display in text "vulnmean{c |}" as result %8.4g `realvv' "  ["
      %7.3g `vn_10l' ","%7.3g `vn_10h' "] ["%7.3g `vn_5l' ","%7.3g `vn_5h'
      "] ["%7.3g `vn_1l' "," %7.3g `vn_1h' "]" ;
    display in text " povmean{c |}" as result %8.4g `realpp' "  ["
      %7.3g `pn_10l' ","%7.3g `pn_10h' "] ["%7.3g `pn_5l' ","%7.3g `pn_5h' 
      "] ["%7.3g `pn_1l' "," %7.3g `pn_1h' "]" ;
    display in text "riskmean{c |}" as result %8.4g `realrr' "  ["
      %7.3g `rn_10l' ","%7.3g `rn_10h' "] ["%7.3g `rn_5l' ","%7.3g `rn_5h' 
      "] ["%7.3g `rn_1l' "," %7.3g `rn_1h' "]" ;
    display in text "agrism~n{c |}" as result %8.4g `realaa' "  ["
      %7.3g `an_10l' ","%7.3g `an_10h' "] ["%7.3g `an_5l' ","%7.3g `an_5h' 
      "] ["%7.3g `an_1l' "," %7.3g `an_1h' "]" ;
    display in text "idrism~n{c |}" as result %8.4g `realitt' "  ["
      %7.3g `in_10l' ","%7.3g `in_10h' "] ["%7.3g `in_5l' ","%7.3g `in_5h' 
      "] ["%7.3g `in_1l' "," %7.3g `in_1h' "]" ;
    forvalues num= 1/`ir_n' {;
      display in text "idri`num'm~n{c |}" as result %8.4g `realii`num''
        "  ["%7.3g `i`num'n_10l' ","%7.3g `i`num'n_10h' "] ["%7.3g 
        `i`num'n_5l' ","%7.3g `i`num'n_5h' "] ["%7.3g `i`num'n_1l' "," 
        %7.3g `i`num'n_1h' "]" ;
    };
    display in text "unexri~n{c |}" as result %8.4g `realuu' "  ["
      %7.3g `un_10l' ","%7.3g `un_10h' "] ["%7.3g `un_5l' ","%7.3g `un_5h' 
      "] ["%7.3g `un_1l' "," %7.3g `un_1h' "]" ;
    
    if `runreg'==1 {;
      clear;
      quietly use `regboot1';
      forvalues num2= 1/`ir_n' {;
        merge using `regboot1i`num2'';
        drop _merge;
        merge using `regbootconsti`num2'';
        drop _merge;
      };
      if `rrv_n' >1 {;
        forvalues num=2/`rrv_n' {; 
          merge using `regboot`num'';
          drop _merge;
          forvalues num2= 1/`ir_n' {;
            merge using `regboot`num'i`num2'';
            drop _merge;
          };
        };
      };
      merge using `regbootconst';
      drop _merge;
      gen id=_n;
      tempname n_1l n_1h n_5l n_5h n_10l n_10h;
      quietly summ id;
      local n_max=r(max);
      scalar `n_1l'=round((.005*`n_max'),1);
      scalar `n_1h'=round((.995*`n_max'),1);
      scalar `n_5l'=round((.025*`n_max'),1);
      scalar `n_5h'=round((.975*`n_max'),1);
      scalar `n_10l'=round((.05*`n_max'),1);
      scalar `n_10h'=round((.95*`n_max'),1);


      forvalues num= 1/`rrv_n' {;
        sort rrbeta`num'v;
        quietly replace id=_n;
        summ rrbeta`num'v if id==`n_1l', meanonly;
        local r`num'vn_1l=r(mean);
        summ rrbeta`num'v if id==`n_1h', meanonly;
        local r`num'vn_1h=r(mean);
        summ rrbeta`num'v if id==`n_5l', meanonly;
        local r`num'vn_5l=r(mean);
        summ rrbeta`num'v if id==`n_5h', meanonly;
        local r`num'vn_5h=r(mean);
        summ rrbeta`num'v if id==`n_10l', meanonly;
        local r`num'vn_10l=r(mean);
        summ rrbeta`num'v if id==`n_10h', meanonly;
        local r`num'vn_10h=r(mean);
      };
      sort rrbetaconstv;
      quietly replace id=_n;
      summ rrbetaconstv if id==`n_1l', meanonly;
      local rconstvn_1l=r(mean);
      summ rrbetaconstv if id==`n_1h', meanonly;
      local rconstvn_1h=r(mean);
      summ rrbetaconstv if id==`n_5l', meanonly;
      local rconstvn_5l=r(mean);
      summ rrbetaconstv if id==`n_5h', meanonly;
      local rconstvn_5h=r(mean);
      summ rrbetaconstv if id==`n_10l', meanonly;
      local rconstvn_10l=r(mean);
      summ rrbetaconstv if id==`n_10h', meanonly;
      local rconstvn_10h=r(mean);

    
      display"";
      display"";
      display as text "Bootstrapped vulnerability regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Nv';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2v'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "Vuln~y{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num= 1/`rrv_n' {;
        display in text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'v' 
          " ["%7.3g `r`num'vn_10l' " ,"%7.3g `r`num'vn_10h' "] ["%7.3g 
          `r`num'vn_5l' " ,"%7.3g `r`num'vn_5h' "] ["%7.3g `r`num'vn_1l' " ," 
          %7.3g `r`num'vn_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconstv'
        " ["%7.3g `rconstvn_10l' " ,"%7.3g `rconstvn_10h' "] ["%7.3g 
        `rconstvn_5l' " ,"%7.3g `rconstvn_5h' "] ["%7.3g `rconstvn_1l' " ," 
        %7.3g `rconstvn_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";


      forvalues num= 1/`rrv_n' {;
        sort rrbeta`num'p;
        quietly replace id=_n;
        summ rrbeta`num'p if id==`n_1l', meanonly;
        local r`num'pn_1l=r(mean);
        summ rrbeta`num'p if id==`n_1h', meanonly;
        local r`num'pn_1h=r(mean);
        summ rrbeta`num'p if id==`n_5l', meanonly;
        local r`num'pn_5l=r(mean);
        summ rrbeta`num'p if id==`n_5h', meanonly;
        local r`num'pn_5h=r(mean);
        summ rrbeta`num'p if id==`n_10l', meanonly;
        local r`num'pn_10l=r(mean);
        summ rrbeta`num'p if id==`n_10h', meanonly;
        local r`num'pn_10h=r(mean);
      };
      sort rrbetaconstp;
      quietly replace id=_n;
      summ rrbetaconstp if id==`n_1l', meanonly;
      local rconstpn_1l=r(mean);
      summ rrbetaconstp if id==`n_1h', meanonly;
      local rconstpn_1h=r(mean);
      summ rrbetaconstp if id==`n_5l', meanonly;
      local rconstpn_5l=r(mean);
      summ rrbetaconstp if id==`n_5h', meanonly;
      local rconstpn_5h=r(mean);
      summ rrbetaconstp if id==`n_10l', meanonly;
      local rconstpn_10l=r(mean);
      summ rrbetaconstp if id==`n_10h', meanonly;
      local rconstpn_10h=r(mean);

    
      display"";
      display"";
      display as text "Bootstrapped poverty regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Nv';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2p'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "Pov~ty{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num= 1/`rrv_n' {;
        display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'p' 
          " ["%7.3g `r`num'pn_10l' " ,"%7.3g `r`num'pn_10h' "] ["%7.3g 
          `r`num'pn_5l' " ,"%7.3g `r`num'pn_5h' "] ["%7.3g `r`num'pn_1l' " ," 
          %7.3g `r`num'pn_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconstp'
        " ["%7.3g `rconstpn_10l' " ,"%7.3g `rconstpn_10h' "] ["%7.3g 
        `rconstpn_5l' " ,"%7.3g `rconstpn_5h' "] ["%7.3g `rconstpn_1l' " ," 
        %7.3g `rconstpn_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";


      forvalues num= 1/`rrv_n' {;
        sort rrbeta`num'r;
        quietly replace id=_n;
        summ rrbeta`num'r if id==`n_1l', meanonly;
        local r`num'rn_1l=r(mean);
        summ rrbeta`num'r if id==`n_1h', meanonly;
        local r`num'rn_1h=r(mean);
        summ rrbeta`num'r if id==`n_5l', meanonly;
        local r`num'rn_5l=r(mean);
        summ rrbeta`num'r if id==`n_5h', meanonly;
        local r`num'rn_5h=r(mean);
        summ rrbeta`num'r if id==`n_10l', meanonly;
        local r`num'rn_10l=r(mean);
        summ rrbeta`num'r if id==`n_10h', meanonly;
        local r`num'rn_10h=r(mean);
      };
      sort rrbetaconstr;
      quietly replace id=_n;
      summ rrbetaconstr if id==`n_1l', meanonly;
      local rconstrn_1l=r(mean);
      summ rrbetaconstr if id==`n_1h', meanonly;
      local rconstrn_1h=r(mean);
      summ rrbetaconstr if id==`n_5l', meanonly;
      local rconstrn_5l=r(mean);
      summ rrbetaconstr if id==`n_5h', meanonly;
      local rconstrn_5h=r(mean);
      summ rrbetaconstr if id==`n_10l', meanonly;
      local rconstrn_10l=r(mean);
      summ rrbetaconstr if id==`n_10h', meanonly;
      local rconstrn_10h=r(mean);


      display"";
      display"";
      display as text "Bootstrapped risk regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Nv';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2r'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "  Risk{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num =1/`rrv_n' {;
        display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'r' 
          " ["%7.3g `r`num'rn_10l' " ,"%7.3g `r`num'rn_10h' "] ["%7.3g 
          `r`num'rn_5l' " ,"%7.3g `r`num'rn_5h' "] ["%7.3g `r`num'rn_1l' " ," 
          %7.3g `r`num'rn_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconstr'
        " ["%7.3g `rconstrn_10l' " ,"%7.3g `rconstrn_10h' "] ["%7.3g 
        `rconstrn_5l' " ,"%7.3g `rconstrn_5h' "] ["%7.3g `rconstrn_1l' " ," 
        %7.3g `rconstrn_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";


      forvalues num= 1/`rrv_n' {;
        sort rrbeta`num'a;
        quietly replace id=_n;
        summ rrbeta`num'a if id==`n_1l', meanonly;
        local r`num'an_1l=r(mean);
        summ rrbeta`num'a if id==`n_1h', meanonly;
        local r`num'an_1h=r(mean);
        summ rrbeta`num'a if id==`n_5l', meanonly;
        local r`num'an_5l=r(mean);
        summ rrbeta`num'a if id==`n_5h', meanonly;
        local r`num'an_5h=r(mean);
        summ rrbeta`num'a if id==`n_10l', meanonly;
        local r`num'an_10l=r(mean);
        summ rrbeta`num'a if id==`n_10h', meanonly;
        local r`num'an_10h=r(mean);
      };
      sort rrbetaconsta;
      quietly replace id=_n;
      summ rrbetaconsta if id==`n_1l', meanonly;
      local rconstan_1l=r(mean);
      summ rrbetaconsta if id==`n_1h', meanonly;
      local rconstan_1h=r(mean);
      summ rrbetaconsta if id==`n_5l', meanonly;
      local rconstan_5l=r(mean);
      summ rrbetaconsta if id==`n_5h', meanonly;
      local rconstan_5h=r(mean);
      summ rrbetaconsta if id==`n_10l', meanonly;
      local rconstan_10l=r(mean);
      summ rrbetaconsta if id==`n_10h', meanonly;
      local rconstan_10h=r(mean);

    
      display"";
      display"";
      display as text "Bootstrapped aggregate risk regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Nv';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2a'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "AggRis{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num= 1/`rrv_n' {;
        display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'a' 
          " ["%7.3g `r`num'an_10l' " ,"%7.3g `r`num'an_10h' "] ["%7.3g 
          `r`num'an_5l' " ,"%7.3g `r`num'an_5h' "] ["%7.3g `r`num'an_1l' " ," 
          %7.3g `r`num'an_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconsta'
        " ["%7.3g `rconstan_10l' " ,"%7.3g `rconstan_10h' "] ["%7.3g 
        `rconstan_5l' " ,"%7.3g `rconstan_5h' "] ["%7.3g `rconstan_1l' " ," 
        %7.3g `rconstan_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";

    
      forvalues num =1/`rrv_n' {;
        sort rrbeta`num'it;
        quietly replace id=_n;
        summ rrbeta`num'it if id==`n_1l', meanonly;
        local r`num'itn_1l=r(mean);
        summ rrbeta`num'it if id==`n_1h', meanonly;
        local r`num'itn_1h=r(mean);
        summ rrbeta`num'it if id==`n_5l', meanonly;
        local r`num'itn_5l=r(mean);
        summ rrbeta`num'it if id==`n_5h', meanonly;
        local r`num'itn_5h=r(mean);
        summ rrbeta`num'it if id==`n_10l', meanonly;
        local r`num'itn_10l=r(mean);
        summ rrbeta`num'it if id==`n_10h', meanonly;
        local r`num'itn_10h=r(mean);
      };
      sort rrbetaconstit;
      quietly replace id=_n;
      summ rrbetaconstit if id==`n_1l', meanonly;
      local rconstitn_1l=r(mean);
      summ rrbetaconstit if id==`n_1h', meanonly;
      local rconstitn_1h=r(mean);
      summ rrbetaconstit if id==`n_5l', meanonly;
      local rconstitn_5l=r(mean);
      summ rrbetaconstit if id==`n_5h', meanonly;
      local rconstitn_5h=r(mean);
      summ rrbetaconstit if id==`n_10l', meanonly;
      local rconstitn_10l=r(mean);
      summ rrbetaconstit if id==`n_10h', meanonly;
      local rconstitn_10h=r(mean);

      display"";
      display"";
      display as text "Bootstrapped total idiosyncratic risk regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Ni';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2it'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "IdRisk{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num= 1/`rrv_n' {;
        display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'it' 
          " ["%7.3g `r`num'itn_10l' " ,"%7.3g `r`num'itn_10h' "] ["%7.3g 
          `r`num'itn_5l' " ,"%7.3g `r`num'itn_5h' "] ["%7.3g `r`num'itn_1l' 
          " ," %7.3g `r`num'itn_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconstit'
        " ["%7.3g `rconstitn_10l' " ,"%7.3g `rconstitn_10h' "] ["%7.3g 
        `rconstitn_5l' " ,"%7.3g `rconstitn_5h' "] ["%7.3g `rconstitn_1l' " ," 
        %7.3g `rconstitn_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";
  

      forvalues num2= 1/`ir_n' {;
        forvalues num= 1/`rrv_n' {;
          sort rrbeta`num'i`num2';
          quietly replace id=_n;
          summ rrbeta`num'i`num2' if id==`n_1l', meanonly;
          local r`num'i`num2'n_1l=r(mean);
          summ rrbeta`num'i`num2' if id==`n_1h', meanonly;
          local r`num'i`num2'n_1h=r(mean);
          summ rrbeta`num'i`num2' if id==`n_5l', meanonly;
          local r`num'i`num2'n_5l=r(mean);
          summ rrbeta`num'i`num2' if id==`n_5h', meanonly;
          local r`num'i`num2'n_5h=r(mean);
          summ rrbeta`num'i`num2' if id==`n_10l', meanonly;
          local r`num'i`num2'n_10l=r(mean);
          summ rrbeta`num'i`num2' if id==`n_10h', meanonly;
          local r`num'i`num2'n_10h=r(mean);
        };
        sort rrbetaconsti`num2';
        quietly replace id=_n;
        summ rrbetaconsti`num2' if id==`n_1l', meanonly;
        local rconsti`num2'n_1l=r(mean);
        summ rrbetaconsti`num2' if id==`n_1h', meanonly;
        local rconsti`num2'n_1h=r(mean);
        summ rrbetaconsti`num2' if id==`n_5l', meanonly;
        local rconsti`num2'n_5l=r(mean);
        summ rrbetaconsti`num2' if id==`n_5h', meanonly;
        local rconsti`num2'n_5h=r(mean);
        summ rrbetaconsti`num2' if id==`n_10l', meanonly;
        local rconsti`num2'n_10l=r(mean);
        summ rrbetaconsti`num2' if id==`n_10h', meanonly;
        local rconsti`num2'n_10h=r(mean);

    
        display"";
        display"";
        display as text "Bootstrapped idiosyncratic risk #" `num2' " regression";
        display as text "                                                 "
          " No. of obs =" as result %8.0g `Ni';
        display as text "                                                 "
          " R-squared  =" as result %8.4g `r2i`num2''; 
        display as text "{hline 6}{c TT}{hline 63}";
        display in text "IdRis" `num2' "{c |} Coef.  [ 90% "
          "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
        display in text "{hline 6}{c +}{hline 63}";
        forvalues num= 1/`rrv_n' {;
          display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
            as result %7.3g `realb`num'i`num2'' 
            " ["%7.3g `r`num'i`num2'n_10l' " ,"%7.3g `r`num'i`num2'n_10h' 
            "] ["%7.3g `r`num'i`num2'n_5l' " ,"%7.3g `r`num'i`num2'n_5h' 
            "] ["%7.3g `r`num'i`num2'n_1l' " ," %7.3g `r`num'i`num2'n_1h' "]" ;
        };
        display as text %6s "_cons""{c |}" as result %7.3g 
          `realbconsti`num2''
          " ["%7.3g `rconsti`num2'n_10l' " ,"%7.3g `rconsti`num2'n_10h' 
          "] ["%7.3g `rconsti`num2'n_5l' " ,"%7.3g `rconsti`num2'n_5h' 
          "] ["%7.3g `rconsti`num2'n_1l' " ," %7.3g `rconsti`num2'n_1h' "]" ;
        display as text "{hline 6}{c BT}{hline 63}";
      };
 

      forvalues num= 1/`rrv_n' {;
        sort rrbeta`num'u;
        quietly replace id=_n;
        summ rrbeta`num'u if id==`n_1l', meanonly;
        local r`num'un_1l=r(mean);
        summ rrbeta`num'u if id==`n_1h', meanonly;
        local r`num'un_1h=r(mean);
        summ rrbeta`num'u if id==`n_5l', meanonly;
        local r`num'un_5l=r(mean);
        summ rrbeta`num'u if id==`n_5h', meanonly;
        local r`num'un_5h=r(mean);
        summ rrbeta`num'u if id==`n_10l', meanonly;
        local r`num'un_10l=r(mean);
        summ rrbeta`num'u if id==`n_10h', meanonly;
        local r`num'un_10h=r(mean);
      };
  
      sort rrbetaconstu;
      quietly replace id=_n;
      summ rrbetaconstu if id==`n_1l', meanonly;
      local rconstun_1l=r(mean);
      summ rrbetaconstu if id==`n_1h', meanonly;
      local rconstun_1h=r(mean);
      summ rrbetaconstu if id==`n_5l', meanonly;
      local rconstun_5l=r(mean);
      summ rrbetaconstu if id==`n_5h', meanonly;
      local rconstun_5h=r(mean);
      summ rrbetaconstu if id==`n_10l', meanonly;
      local rconstun_10l=r(mean);
      summ rrbetaconstu if id==`n_10h', meanonly;
      local rconstun_10h=r(mean);

    
      display"";
      display"";
      display as text "Bootstrapped unexplained risk regression";
      display as text "                                                 "
        " No. of obs =" as result %8.0g `Ni';
      display as text "                                                 "
        " R-squared  =" as result %8.4g `r2u'; 
      display as text "{hline 6}{c TT}{hline 63}";
      display in text "UnxRis{c |} Coef.  [ 90% "
        "Conf. Int. ] [ 95% Conf. Int. ] [ 99% Conf. Int. ]";
      display in text "{hline 6}{c +}{hline 63}";
      forvalues num= 1/`rrv_n' {;
        display as text %6s abbrev("${namerrv`num'}",6) "{c |}"
          as result %7.3g `realb`num'u' 
          " ["%7.3g `r`num'un_10l' " ,"%7.3g `r`num'un_10h' "] ["%7.3g 
          `r`num'un_5l' " ,"%7.3g `r`num'un_5h' "] ["%7.3g `r`num'un_1l' " ," 
          %7.3g `r`num'un_1h' "]" ;
      };
      display as text %6s "_cons""{c |}" as result %7.3g `realbconstu'
        " ["%7.3g `rconstun_10l' " ,"%7.3g `rconstun_10h' "] ["%7.3g 
        `rconstun_5l' " ,"%7.3g `rconstun_5h' "] ["%7.3g `rconstun_1l' " ," 
        %7.3g `rconstun_1h' "]" ;
      display as text "{hline 6}{c BT}{hline 63}";
      restore;
    };
  };


  if `boot'==0 & `runreg'==1 {;
    display"";
    display"";
    display as text "Non-bootstrapped vulnerability regression"; 
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Nv;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2v; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "   Vulnerability{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num= 1/`rrv_n' {;
      display in text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'v';
    };
    display as text %16s "_cons""{c |}" as result %7.3g `realbconstv';
    display as text "{hline 16}{c BT}{hline 53}";
    
    display"";
    display"";
    display as text "Non-bootstrapped poverty regression";
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Nv;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2p; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "         Poverty{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num= 1/`rrv_n' {;
      display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'p';
    };
    display as text %16s "_cons""{c |}" as result %8.4g `realbconstp';
    display as text "{hline 16}{c BT}{hline 53}";

    display"";
    display"";
    display as text "Non-bootstrapped risk regression";
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Nv;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2r; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "            Risk{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num =1/`rrv_n' {;
      display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'r';
    };
    display as text %16s "_cons""{c |}" as result %8.4g `realbconstr';
    display as text "{hline 16}{c BT}{hline 53}";


    
    display"";
    display"";
    display as text "Non-bootstrapped aggregate risk regression";
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Nv;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2a; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "  Aggregate Risk{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num= 1/`rrv_n' {;
      display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'a';
    };
    display as text %16s "_cons""{c |}" as result %8.4g `realbconsta';
    display as text "{hline 16}{c BT}{hline 53}";

    display"";
    display"";
    display as text "Non-bootstrapped total idiosyncratic risk regression";
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Ni;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2it; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "  Idiosync. Risk{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num= 1/`rrv_n' {;
      display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'it';
    };
    display as text %16s "_cons""{c |}" as result %8.4g `realbconstit';
    display as text "{hline 16}{c BT}{hline 53}";
  

    forvalues num2=1/`ir_n' {;   
      display"";
      display"";
      display as text "Non-bootstrapped idiosyncratic risk #" `num2' " regression";
      display"";
      display as text "                                                 "
        " No. of obs =" as result %8.0g $Ni;
      display as text "                                                 "
        " R-squared  =" as result %8.4g ${r2i`num2'}; 
      display as text "{hline 16}{c TT}{hline 53}";
      display in text "Idiosyn. Risk #" `num2' "{c |} Coefficient";
      display in text "{hline 16}{c +}{hline 53}";
      forvalues num= 1/`rrv_n' {;
        display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
          as result %8.4g `realb`num'i`num2'';
      };
      display as text %16s "_cons""{c |}" as result %8.4g 
        `realbconsti`num2'';
      display as text "{hline 16}{c BT}{hline 53}";
    };
  
    display"";
    display"";
    display as text "Non-bootstrapped unexplained risk regression";
    display"";
    display as text "                                                 "
      " No. of obs =" as result %8.0g $Ni;
    display as text "                                                 "
      " R-squared  =" as result %8.4g $r2u; 
    display as text "{hline 16}{c TT}{hline 53}";
    display in text "Unexplained Risk{c |} Coefficient";
    display in text "{hline 16}{c +}{hline 53}";
    forvalues num= 1/`rrv_n' {;
      display as text %16s abbrev("${namerrv`num'}",16) "{c |}"
        as result %8.4g `realb`num'u';
    };
    display as text %16s "_cons""{c |}" as result %8.4g `realbconstu';
    display as text "{hline 16}{c BT}{hline 53}";
  };
end;


